#!/usr/bin/env python


"""
RCSid:
	$Id: OpenSSLSigner.py,v 1.8 2013/07/18 20:41:11 sjg Exp $

	@(#) Copyright (c) 2012 Simon J. Gerraty

	This file is provided in the hope that it will
	be of use.  There is absolutely NO WARRANTY.
	Permission to copy, redistribute or otherwise
	use this file is hereby granted provided that 
	the above copyright notice and this notice are
	left intact. 
      
	Please send copies of changes and bug-fixes to:
	sjg@crufty.net

"""

from Signer import *
import hashlib, os, sys
sys.path.append(host_target())
import signer as ossl

class OpenSSLSigner(Signer):
    """Interface to generic signing routine"""
    def __init__(self, kfile, conf={}):
        self.pubkey = None
        Signer.__init__(self, kfile, conf)
        self.ktype = getConfig(conf, 'KeyType')
        self.hname = getConfig(conf, 'Hash', 'sha256')
        self.pubkfile = getConfig(conf, 'PublicKey')
        self.pubkey = None
        if self.hname.startswith('fake'):
            import FakeHash
            self.hash = self.fake_hash
            self.hash_new = FakeHash.FakeHash
            self.hash_size = 20
            if self.hname.endswith('256'):
                self.hash_size = 32
        else:
            m = __import__('hashlib', fromlist=[self.hname])
            self.hash = getattr(m, self.hname)

        # any engine support should be setup before we load_key
        self.engine = getConfig(conf, 'Engine')
        if self.engine:
            ossl.use_engine(self.engine)

        if not self.pubkfile:
            self.load_key()
            

    def load_key(self):
        self.key = ossl.load_key(self.kfile)
        if not self.key:
            raise ValueError('cannot load key from {0}'.format(self.kfile))

    def load_pubkey(self):
        self.pubkey = ossl.load_pubkey(self.pubkfile)
        if not self.pubkey:
            raise ValueError('cannot load public key from {0}'.format(self.pubkfile))

    def fake_hash(self, msg=None):
        return self.hash_new(msg, digest_size=self.hash_size)

    def sign(self, msg):
        h = self.hash(msg)
        return ossl.sign_digest(self.key, self.hname, h.digest())

    def verify(self, mdata, sig):
        """verify sig against mdata"""
        if not self.pubkey:
            self.load_pubkey()
        return ossl.verify_digest(self.pubkey, self.hname, mdata, sig)


if __name__ == '__main__':
    import getopt, sys

    kfile = '/tmp/the.key'
    sfile = '/tmp/the.osig'
    mfile = '/tmp/the'

    conf = {'Hash': 'fakesha256'}
    opts,args = getopt.getopt(sys.argv[1:], 'k:m:s:')
    for o,a in opts:
        if o == '-k':
            kfile = a
        elif o == '-m':
            mfile = a
        elif o == '-s':
            sfile = a

    signer = OpenSSLSigner(kfile, conf)
    msg = open(mfile, 'r').read()
    h = hashlib.sha256(msg)
    sig = signer.sign(h.hexdigest())
    try:
        if signer.verify(h.hexdigest(), sig):
            print "Verified"
    except:
        pass
    print pem_encode('SIGNATURE', sig)

